@@ -2,9 +2,7 @@ from django.contrib.auth.models import User
 from django.core import mail
 from django.core.urlresolvers import reverse
 from django.test import TestCase
-from django.test.client import RequestFactory
 
-from .. import views
 from ..forms import PasswordRecoveryForm, PasswordResetForm
 
 
@@ -137,8 +135,11 @@ class ViewTests(TestCase):
         self.assertContains(response, "Sorry, this user")
 
         self.assertEqual(len(mail.outbox), 0)
-        response = self.client.post(url, {'username_or_email': 'foo'})
-        self.assertContains(response, "<strong>bar@example.com</strong>")
+        response = self.client.post(url, {'username_or_email': 'foo'},
+                                    follow=True)
+        self.assertEqual(len(response.redirect_chain), 1)
+        self.assertContains(response, 'bar@example.com')
+
         self.assertEqual(len(mail.outbox), 1)
 
         message = mail.outbox[0]
@@ -183,10 +184,12 @@ class ViewTests(TestCase):
         self.assertContains(response, "Sorry, this user")
 
         self.assertEqual(len(mail.outbox), 0)
-        response = self.client.post(url,
-                                    {'username_or_email': 'bar@example.com'})
+        response = self.client.post(
+            url, {'username_or_email': 'bar@example.com'}, follow=True,
+        )
         self.assertEqual(len(mail.outbox), 1)
-        self.assertContains(response, '<strong>bar@example.com</strong>')
+        self.assertEqual(len(response.redirect_chain), 1)
+        self.assertContains(response, 'bar@example.com')
 
     def test_username_recover(self):
         url = reverse('username_recover')
@@ -200,7 +203,60 @@ class ViewTests(TestCase):
         self.assertContains(response, "Sorry, this user")
 
         self.assertEqual(len(mail.outbox), 0)
-        response = self.client.post(url,
-                                    {'username_or_email': 'foo'})
+        response = self.client.post(
+            url, {'username_or_email': 'foo'}, follow=True,
+        )
         self.assertEqual(len(mail.outbox), 1)
+        self.assertEqual(len(response.redirect_chain), 1)
+        self.assertContains(response, 'foo')
+
+    def test_invalid_signature(self):
+        url = reverse('password_reset_sent',
+                      kwargs={'signature': 'test@example.com:122323333'})
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 404)
+
+    def test_content_redirection(self):
+        url = reverse('username_recover')
+        response = self.client.get(url)
+
+        response = self.client.post(
+            url, {'username_or_email': 'foo'}, follow=True,
+        )
+        self.assertEqual(len(mail.outbox), 1)
+        self.assertEqual(len(response.redirect_chain), 1)
         self.assertContains(response, '<strong>foo</strong>')
+        url = reverse('email_recover')
+        response = self.client.post(
+            url, {'username_or_email': 'bar@example.com'}, follow=True,
+        )
+        self.assertEqual(len(mail.outbox), 2)
+        self.assertEqual(len(response.redirect_chain), 1)
+        self.assertContains(response, '<strong>bar@example.com</strong>')
+
+    def test_insensitive_recover(self):
+        url = reverse('insensitive_recover')
+        response = self.client.get(url)
+        normalized = '<strong>bar@example.com</strong>'
+
+        self.assertContains(response, 'Username or Email')
+        self.assertEqual(len(mail.outbox), 0)
+        response = self.client.post(url, {'username_or_email': 'FOO'},
+                                    follow=True)
+        self.assertEqual(len(mail.outbox), 1)
+        self.assertEqual(len(response.redirect_chain), 1)
+        self.assertContains(response, normalized)
+
+        response = self.client.post(
+            url, {'username_or_email': 'bar@EXAmPLE.coM'}, follow=True,
+        )
+        self.assertEqual(len(mail.outbox), 2)
+        self.assertEqual(len(response.redirect_chain), 1)
+        self.assertContains(response, normalized)
+
+        response = self.client.post(
+            url, {'username_or_email': 'bar@example.com'}, follow=True,
+        )
+        self.assertEqual(len(mail.outbox), 3)
+        self.assertEqual(len(response.redirect_chain), 1)
+        self.assertContains(response, normalized)
