@@ -1,11 +1,15 @@
+import datetime
+
 from django.conf import settings
 from django.contrib.auth.models import User
 from django.contrib.sites.models import RequestSite
 from django.core import signing
 from django.core.mail import send_mail
-from django.core.urlresolvers import reverse_lazy
+from django.core.urlresolvers import reverse, reverse_lazy
 from django.shortcuts import get_object_or_404, redirect
+from django.http import Http404
 from django.template import loader
+from django.utils import timezone
 from django.views import generic
 
 from .forms import PasswordRecoveryForm, PasswordResetForm
@@ -13,6 +17,33 @@ from .forms import PasswordRecoveryForm, PasswordResetForm
 
 class SaltMixin(object):
     salt = 'password_recovery'
+    url_salt = 'password_recovery_url'
+
+
+def loads_with_timestamp(value, salt):
+    """Returns the unsigned value along with its timestamp, the time when it
+    got dumped."""
+    try:
+        data = signing.loads(value, salt=salt, max_age=-1)
+    except signing.SignatureExpired as e:
+        age = float(str(e).split('Signature age ')[1].split(' >')[0])
+        timestamp = timezone.now() - datetime.timedelta(seconds=age)
+    return timestamp, signing.loads(value, salt=salt)
+
+
+class RecoverDone(SaltMixin, generic.TemplateView):
+    template_name = "password_reset/reset_sent.html"
+
+    def get_context_data(self, **kwargs):
+        ctx = super(RecoverDone, self).get_context_data(**kwargs)
+        try:
+            ctx['timestamp'], ctx['email'] = loads_with_timestamp(
+                self.kwargs['signature'], salt=self.url_salt,
+            )
+        except signing.BadSignature:
+            raise Http404
+        return ctx
+recover_done = RecoverDone.as_view()
 
 
 class Recover(SaltMixin, generic.FormView):
@@ -23,6 +54,9 @@ class Recover(SaltMixin, generic.FormView):
     email_subject_template_name = 'password_reset/recovery_email_subject.txt'
     search_fields = ['username', 'email']
 
+    def get_success_url(self):
+        return reverse('password_reset_sent', args=[self.mail_signature])
+
     def get_context_data(self, **kwargs):
         kwargs['url'] = self.request.get_full_path()
         return super(Recover, self).get_context_data(**kwargs)
@@ -59,8 +93,8 @@ class Recover(SaltMixin, generic.FormView):
             email = self.user.username
         else:
             email = self.user.email
-        context = {'email': email}
-        return self.render_to_response(self.get_context_data(**context))
+        self.mail_signature = signing.dumps(email, salt=self.url_salt)
+        return super(Recover, self).form_valid(form)
 recover = Recover.as_view()
 
 
@@ -109,4 +143,6 @@ reset = Reset.as_view()
 
 class ResetDone(generic.TemplateView):
     template_name = 'password_reset/recovery_done.html'
+
+
 reset_done = ResetDone.as_view()
